#include "test.h"
#include "queue.h"

int test_free() {
  struct Queue *q = queue_malloc(4);
  queue_free(q);
  return 0;
}

int test_add() {
  struct Queue *q = queue_malloc(4);
  unsigned char c;
  test_assert(!queue_add(q, 'a'));
  test_assert(!queue_add(q, 'b'));
  test_assert(!queue_add(q, 'c'));
  test_assert(!queue_add(q, 'd'));
  test_assert(queue_add(q, 'x'));
  queue_print(q);

  queue_remove(q, &c);
  queue_remove(q, &c);
  test_assert(!queue_add(q, 'e'));
  test_assert(!queue_add(q, 'f'));
  test_assert(queue_add(q, 'x'));
  queue_print(q);

  return 0;
}

int test_remove() {
  int failed = 0;
  struct Queue *q = queue_malloc(4);
  unsigned char c;
  queue_add(q, 'x');
  queue_add(q, 'x');
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'x'));
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'x'));
  test_assert(queue_remove(q, &c));
  queue_print(q);

  queue_add(q, 'a');
  queue_add(q, 'b');
  queue_add(q, 'c');
  queue_add(q, 'd');
  queue_add(q, 'x');
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'a'));
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'b'));
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'c'));
  test_assert(!queue_remove(q, &c));
  test_assert((c == 'd'));
  test_assert(queue_remove(q, &c));

  return failed;
}


int test_size() {
  int failed = 0;
  struct Queue *q = queue_malloc(4);
  unsigned char c;

  test_assert(queue_size(q) == 0);
  queue_print(q);

  queue_add(q, 'a');
  test_assert(queue_size(q) == 1);
  queue_print(q);

  queue_add(q, 'b');
  test_assert(queue_size(q) == 2);
  queue_print(q);

  queue_add(q, 'c');
  test_assert(queue_size(q) == 3);
  queue_print(q);

  queue_add(q, 'd');
  test_assert(queue_size(q) == 4);
  queue_print(q);

  queue_remove(q, &c);
  queue_remove(q, &c);
  test_assert(queue_size(q) == 2);
  queue_print(q);

  queue_add(q, 'e');
  test_assert(queue_size(q) == 3);
  queue_print(q);

  return failed;
}

int test_full() {
  int failed = 0;
  struct Queue *q = queue_malloc(4);
  unsigned char c;

  test_assert(!queue_full(q));
  queue_print(q);

  queue_add(q, 'a');
  queue_add(q, 'b');
  queue_add(q, 'c');
  test_assert(!queue_full(q));
  queue_print(q);

  queue_add(q, 'd');
  test_assert(queue_full(q));
  queue_print(q);

  queue_remove(q, &c);
  test_assert(!queue_full(q));
  queue_print(q);

  queue_remove(q, &c);
  queue_add(q, 'x');
  queue_add(q, 'x');
  test_assert(queue_full(q));
  queue_print(q);

  return failed;
}

int test_empty() {
  int failed = 0;
  struct Queue *q = queue_malloc(4);
  unsigned char c;

  test_assert(queue_empty(q));
  queue_print(q);

  queue_add(q, 'a');
  queue_add(q, 'b');
  queue_add(q, 'c');
  test_assert(!queue_empty(q));
  queue_print(q);

  queue_add(q, 'd');
  test_assert(!queue_empty(q));
  queue_print(q);

  queue_remove(q, &c);
  test_assert(!queue_empty(q));
  queue_print(q);

  queue_remove(q, &c);
  queue_remove(q, &c);
  test_assert(!queue_empty(q));

  queue_remove(q, &c);
  test_assert(queue_empty(q));

  return failed;
}


int main() {
  TEST(test_free);
  TEST(test_add);
  TEST(test_remove);
  TEST(test_size);
  TEST(test_full);
  TEST(test_empty);
  return test_summary();
}
